#ifndef STOCK_H

#define STOCK_H

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

int portnum;
size_t resize;
FILE *logfile;

typedef int32_t plid_t;
typedef int32_t cid_t; 
typedef int32_t grid_t;
typedef u_int32_t pr_t;
typedef int32_t qty_t;
typedef u_int8_t req_t;
typedef int32_t passwd_t;
typedef u_int32_t cash_t;


#include "services.h"
#include "socket.h"
#include "players.h"
#include "companies.h"
#include "threads.h"

/* getopts.h */

void init_defaults ();
void parse_options (int ac, char *av[]);


/* utility.c */
void *xmalloc(size_t size);
void *xrealloc (void *p, size_t size);
void signal_stuff ();
void thread_stuff ();


/* companies.c */
void dump_companies();
void init_companies();

/* socket.c */
void sock_read (int socket, void *buf, int cnt);
void sock_write (int socket, void *buf, int cnt);
inline void sw_s_int (int socket, int16_t s);
inline void sw_int (int socket, int32_t i);


/* blackboard.c */
void *serve_requests_sync (void *arg);
void *serve_requests_async (void *arg);
void *trader ();

/* login.c */
plid_t do_login (int client, pthread_t *th);
void do_login2 (plid_t plid);
void kick_player ();

/* comp-info.c */
void send_company_info ();

/* transactions.c */
void buy_pr_quote(int client, plid_t plid, cid_t cid, pr_t pr, qty_t qty);
void sell_pr_quote(int client, plid_t plid, cid_t cid, pr_t pr, qty_t qty);
void buy_pr_quote_change(int client, plid_t plid, cid_t cid, pr_t pr, 
			 qty_t qty);
void sell_pr_quote_change(int client, plid_t plid, cid_t cid, pr_t pr, 
			  qty_t qty);

/* ds-routines.c */

/* insertion sort routines */
void buy_pr_remove (cid_t cid, plid_t plid, int posn);
void buy_pr_insert (cid_t cid, plid_t plid, pr_t pr, qty_t qty);
void sell_pr_remove (cid_t cid, plid_t plid, int posn);
void sell_pr_insert (cid_t cid, plid_t plid, pr_t pr, qty_t qty);
/* hash table manipulation routines */
struct stock_player_s *holding_insert (plid_t plid, cid_t cid, qty_t qty);
struct stock_player_s *holding_search (plid_t plid, cid_t cid);
/* queue routines */
void setup_req_list ();
void setup_result_list (plid_t plid);
void grow_result_list (result_list_t *node, int n);
int addto_req_list (int client, plid_t plid, cid_t cid,
		    void (*call_fn)
		    (int client, plid_t plid, cid_t cid, pr_t pr, qty_t qty));
void addto_result_list(plid_t plid, req_t req, cid_t cid, qty_t qty, pr_t pr);

#endif /* STOCK_H */
